%A script to check the correspondence of the analytical Musker profile with
%its numerical counterpart and the correspondence of general/numerical wall functions
%with the relative Spalart-Allmaras solution (by a modified fv1).
%Note that the Spalart-Allmaras solution will match only if dpdx=0
clear all
close all
clc

%Fluid/flow parameters
mu=1/2000;
rho=1;
Cp=1;
Pr=1;
Prt=1;
dpdx=0;
qdot=0;

%Geometry and bc
H=1;
U0=0;   UH=28;
T0=300; TH=328;

%Musker model constants
a0=10.306;
vk0=0.384;

%Numerical settings for 1d solution
nit=500; %Max number of iterations
nn=2001; %N. of grid nodes

%**************************************************************************

Re = rho*(UH-U0)*H/mu;
Fu = rho*dpdx*H^3/mu^2;
cypl = abs(Fu)^(2/3);
cut  = (mu*abs(dpdx)/rho^2)^(2/3);
qs=-qdot;

%Step 1: Musker analytical and numerical WF + related SA 1d solution
%Analytical WF iterations
[ait,~,nitend] = spiter(Re,Fu,cypl,@(x) muskersp(x,a0,vk0,0,1));
tw = mu^2*ait(nitend)/(rho*H^2);
ut = sqrt(abs(tw)/rho+cut);

%SA 1d solution
nuH=vk0*H*ut;
[yc,~,~,u1,t1,~,~,~,~] = sa1d(mu,rho,Cp,Pr,dpdx,qdot,0,H,U0,UH,0,T0,TH,nuH,nuH,Prt,0,a0,vk0,0,nit,1e-10,0.5,'gratio',1.01,nn);

%Analytical WF profiles
uwf1a=zeros(size(yc));
twf1a=zeros(size(yc));
ypl1a=yc*ut*rho/mu;
for i = 1:length(yc)
    [s,~,~] = muskersp(ypl1a(i),a0,vk0,0,1);
    uwf1a(i) = U0 + (tw*s(1) + dpdx*yc(i)*s(2))*yc(i)/mu;
end
[s,~,~] = muskersp(H*ut*rho/mu,a0,vk0,0,Pr/Prt);
qw = (((TH-T0)/H)*(mu*Cp/Pr)-H*s(2)*qs)/s(1);
for i = 1:length(yc)
    [s,~,~] = muskersp(ypl1a(i),a0,vk0,0,Pr/Prt);
    twf1a(i) = T0 + (qw*s(1) + qs*yc(i)*s(2))*yc(i)*Pr/(mu*Cp);
end

%Numerical WF iterations and profiles
mutmu = @(y,a,vk) (vk*y).^3./((vk*y).^2+(vk*a)^3-(vk*a)^2);
[ait,~,nitend] = spiter(Re,Fu,cypl,@(x) numericalsp(x,@(y) mutmu(y,a0,vk0),0,1));
tw = mu^2*ait(nitend)/(rho*H^2);
ut = sqrt(abs(tw)/rho+cut);

uwf1n=zeros(size(yc));
twf1n=zeros(size(yc));
ypl1n=yc*ut*rho/mu;
for i = 1:length(yc)
    [s,~,~] = numericalsp(ypl1n(i),@(y) mutmu(y,a0,vk0),0,1);
    uwf1n(i) = U0 + (tw*s(1) + dpdx*yc(i)*s(2))*yc(i)/mu;
end
[s,~,~] = numericalsp(H*ut*rho/mu,@(y) mutmu(y,a0,vk0),0,Pr/Prt);
qw = (((TH-T0)/H)*(mu*Cp/Pr)-H*s(2)*qs)/s(1);
for i = 1:length(yc)
    [s,~,~] = numericalsp(ypl1n(i),@(y) mutmu(y,a0,vk0),0,Pr/Prt);
    twf1n(i) = T0 + (qw*s(1) + qs*yc(i)*s(2))*yc(i)*Pr/(mu*Cp);
end

%Step 2: SA numerical WF + related SA 1d solution
%Numerical WF iterations
vk0=0.41;
cv1=7.1;
mutmu = @(y,a,vk) (vk*y).^4./((vk*y).^3+a^3);
[ait,fait,nitend] = spiter(Re,Fu,cypl,@(x) numericalsp(x,@(y) mutmu(y,cv1,vk0),0,1));
tw = mu^2*ait(nitend)/(rho*H^2);
ut = sqrt(abs(tw)/rho+cut);

%SA 1d solution
nuH=vk0*H*ut;
[yc,~,~,u2,t2,~,~,~,~] = sa1d(mu,rho,Cp,Pr,dpdx,qdot,0,H,U0,UH,0,T0,TH,nuH,nuH,Prt,0,0,vk0,0,nit,1e-10,0.5,'gratio',1.01,nn);

%Numerical WF profiles
uwf2=zeros(size(yc));
twf2=zeros(size(yc));
ypl2=yc*ut*rho/mu;
for i = 1:length(yc)
    [s,~,~] = numericalsp(ypl2(i),@(y) mutmu(y,cv1,vk0),0,1);
    uwf2(i) = U0 + (tw*s(1) + dpdx*yc(i)*s(2))*yc(i)/mu;
end
[s,~,~] = numericalsp(H*ut*rho/mu,@(y) mutmu(y,cv1,vk0),0,Pr/Prt);
qw = (((TH-T0)/H)*(mu*Cp/Pr)-H*s(2)*qs)/s(1);
for i = 1:length(yc)
    [s,~,~] = numericalsp(ypl2(i),@(y) mutmu(y,cv1,vk0),0,Pr/Prt);
    twf2(i) = T0 + (qw*s(1) + qs*yc(i)*s(2))*yc(i)*Pr/(mu*Cp);
end

%Errors
disp(strcat('Eu musker(analytical-numerical):',num2str(norm(uwf1a-uwf1n))))
disp(strcat('Et musker(analytical-numerical):',num2str(norm(twf1a-twf1n))))
disp(strcat('Eu musker(numerical-1d):',num2str(norm(uwf1n-u1'))))
disp(strcat('Et musker(numerical-1d):',num2str(norm(twf1n-t1'))))
disp(strcat('Eu sa(numerical-1d):',num2str(norm(uwf2-u2'))))
disp(strcat('Et sa(numerical-1d):',num2str(norm(twf2-t2'))))