function [s,p,q] = muskersp(y,a0,vk0,N,prprt)
%The function returns the non-dimensional functions si/y+^(i+2) and pi/y+^(i+2)
%of the musker profile for given musker parameters a0 and vk0.
%Functions q are instead needed in computing the derivatives for the NR
%iterations
%N is the higher degree in the expansion and should be -1 to exclude the
%non equilibrium terms.
%The thermal case is handled by changing the profile parameters a0 and vk0
%into a and vk trough the optional parameter prprt=pr/pr_t. When the
%paremeter is present, the average kinetic energy production is not computed. 

%0) Let be sure N is actually within allowed bounds (-1<=N) 
N=max(-1,N); %Altough, note that we assign 0 to all the terms for N>0

%1) Check if velocity case
velo = prprt<=0;

%2) Determine new profile parameters for thermal case: a(Pr,Pr_t,a0) and vk(Pr,Pr_t,vk0)
if ~velo
    c0    = vk0*a0;
    theta = 3*c0*prprt;
    phi   = 3*(c0-1);
    psi   = theta*phi/2;
    eta   = (1+theta*(psi+sqrt(psi^2+phi)))^(1/3);
    a     = (1+eta+eta^2)*a0/(theta*eta);
    vk    = prprt*vk0;
else
    a  = a0;
    vk = vk0;
end

%3) Assembling the profiles
s=zeros(1,N+2);
p=zeros(1,2*N+1+2); %Note p dimension bigger than s
q=zeros(1,N+2);

x=vk*y;
c=vk*a;
mutmu=x^3/(x^2+c^3-c^2);

if y<0.001*a %Low y+ case: use known limiting behavior
    %s
    for i=-1:N
        if ~velo
            s(i+2)=1/(i+2)-prprt*x^3/(c^2*(c-1)*(i+5));
        else
            s(i+2)=1/(i+2)-x^3/(c^2*(c-1)*(i+5));
        end
        q(i+2)=-3*vk^2*x/(c^2*(c-1)*(i+5));
    end
    %p
    if velo
        for i=-1:2*N+1
            p(i+2)=x^3/(c^2*(c-1)*(i+5));
        end
    end
else       %General case
    %Several parts of the profiles are common to s and p for all i, we
    %compute them here
    c0=3*c;
    c1=c-1;
    c2=c0-2;
    c3=c0+2;
    c4=1/(c0+1);
    c5=sqrt(c4/c1);
    cc=c^2;
    ccc=cc*c1;
    f1=log((x+c)/c);
    f2=log((c1*(c-x)+x^2)/(c*c1));
    f3=2*c5*c1^2*(atan((2*x-c1)*c5)+atan(c1*c5));

    if velo
        c23=c2^3;
        c24=c2*c4;
        c6=cc^2*(cc-2*c+1);
        c246=c24*c6;
        c7=c24*(19*ccc+4);   %[19c^2(c-1)+4](3c-2)/(3c+1)
        c8=c24*(9*c6+4*ccc); %[ 9c^4(c^2-2c+1)+4c^2(c-1)](3c-2)/(3c+1)
        c9=c8+6*c246 ;       %[15c^4(c^2-2c+1)+4c^2(c-1)](3c-2)/(3c+1)
        c10=c9+4*c246;       %[19c^4(c^2-2c+1)+4c^2(c-1)](3c-2)/(3c+1)
        f4=(x+1)*x^2+ccc;
    end

    %Then we compute the specific parts for each of the functions
    %s
    for i=-1:N
        d=zeros(1,3+i+1+1); %from d(0) to d(3+i+1)
        switch i
            case -1
                d(1)=1/(2*c2);
                d(2)=2*cc;
                d(3)=c2-cc;
                d(4)=c3;
            case 0
                d(1)=1/(2*c2);
                d(2)=-2*cc*c;
                d(3)=c*cc-c2;
                d(4)=3*cc-c3;
                d(5)=2*c2;
            otherwise
                %We don't have other integrals yet.
        end
        j=i+2;
        s(j)=d(1+1)*f1+d(2+1)*f2+d(3+1)*f3;
        for m=1:i+1
            s(j)=s(j)+d(3+m+1)*x^m;
        end
        s(j)=s(j)*d(0+1)/x^j;
        q(j)=(1/(1+mutmu)-j*s(j))/y^2;
    end
    %p
    if velo
        for i=-1:2*N+1
            d=zeros(1,6+i+1+1); %from d(0) to d(6+i+1)
            switch i
                case -1
                    d(1)=1/(2*c23);
                    d(2)=2*cc*(c*c3-4);
                    d(3)=-(3*cc^2-25*cc*(c-2)-4*(9*c-2));
                    d(4)=c4*(9*c*cc*(c0-1)-78*cc+4*c3);
                    d(5)=2*c7;
                    d(6)=-2*c8;
                    d(7)=2*c9;
                case 0
                    d(1)=1/c23;
                    d(2)=-2*cc*c*(3*cc-2);
                    d(3)=c1^2*(3*cc*(c+2)-4*(5*c-2));
                    d(4)=c4*(3*c*(9*cc^2-30*c*cc+7*cc+22*c-4)-8);
                    d(5)=-(c8+c7);
                    d(6)=c9;
                    d(7)=-c10;
                    d(8)=c23;
                case 1
                    d(1)=1/(2*c23);
                    d(2)=2*cc^2*(9*cc-2*c-4);
                    d(3)=c1^2*(18*cc^2-43*c*cc-10*cc+60*c-24);
                    d(4)=-c4*c1*(225*cc^2-219*c*cc-146*cc+60*c+24);
                    d(5)=2*(c9+c8+c7);
                    d(6)=-2*c10;
                    d(7)=2*(c246*(4+9*ccc)+c10);
                    d(8)=-4*c23;
                    d(9)=c23;
                otherwise
                    %We don't have other integrals yet.
            end
            j=i+2;
            p(j)=d(1+1)*f1+d(2+1)*f2+d(3+1)*f3+(d(4+1)*x^2+d(5+1)*x+d(6+1))/f4-d(6+1)/ccc;
            for m=1:i+1
                p(j)=p(j)+d(6+m+1)*x^m;
            end
            p(j)=p(j)*d(0+1)/x^j;
        end
    end
end