function [s,p,q] = numericalsp(y,mutmu,N,prprt)
%The function returns the non-dimensional functions si/y+^(i+2) and pi/y+^(i+2)
%for the given turbulent viscosity ratio formula by numerical integration.
%Functions q are instead needed in computing the derivatives for the NR
%iterations
%N is the higher degree in the expansion and should be -1 to exclude the
%non equilibrium terms.
%The thermal case is handled by passing in prprt=pr/pr_t. When the
%paremeter is present, the average kinetic energy production is not computed. 

%0) Let be sure N is actually within allowed bounds (-1<=N)
N=max(-1,N);

%1) Check if velocity case
velo = prprt<=0;
if ~velo
    k=prprt;
else
    k=1;
end

%2) Assembling the profiles
s=zeros(1,N+2);
p=zeros(1,2*N+1+2); %Note p dimension bigger than s
q=zeros(1,N+2);

if y<0.01 %Low y+ case: use known limiting behavior
    %We need to test the turbulent viscosity for its limiting behavior
    if (mutmu(0)~=0)
        error('Turbulent viscosity ratio must be 0 for y+=0')
    else
        %We assume mut/mu behaves like cy^n for y->0
        %We need to determine C and n
        y1=0.00001; y2=2*y1;
        f2=mutmu(y2); f1=mutmu(y1);
        if (f1>0)
            n=round(log(f2/f1)/log(y2/y1));
            if (n<2)
                error('Turbulent viscosity goes to 0 too slow (n<2) for y+->0')
            else
                C=(f1-f2)/(y1^n-y2^n);
                if (C<=0)
                    error('Could not reliably assess the turbulent viscosity behavior for y->0')
                end
            end
        else
            %Turbulent viscosity is 0 for non 0 y, derivative is 0
            C=0;n=2;
        end
    end
    %s
    for i=-1:N
        s(i+2)=1/(i+2)-k*C*y^n/(i+n+2);
        q(i+2)=-n*C*y^(n-2)/(i+n+2);
    end
    %p
    if velo
        for i=-1:2*N+1
            p(i+2)=C*y^n/(i+n+2);
        end
    end
else       %General case
    %s
    for i=-1:N
        j=i+2;
        integrand = @(x) x.^(i+1)./(1+k*mutmu(x));
        s(j)=integral(integrand,0,y)/y^j;
        q(j)=(1/(1+mutmu(y))-j*s(j))/y^2;
    end
    %p
    if velo
        for i=-1:2*N+1
            j=i+2;
            integrand = @(x) x.^(i+1).*mutmu(x)./(1+mutmu(x)).^2;
            p(j)=integral(integrand,0,y)/y^j;
        end
    end
end