function [yc,yn,dist,u,t,nut,mut,res,it] = sa1d(mu,rho,Cp,Pr,dpdx,qdot,y0,H,U0,UH,fbcH,T0,TH,nuH,nu0,Prt,sabcH,a0,vk,iwall,nit,toll,saurf,method,par,nn)
%Function to solve the 1D TBLE with the Spalart-Allmaras turbulence model
%for y0<=y<=y0+H, with the bc at y0 always of type wall.
%Uses a cell-centered 2nd order finite volume discretization and the
%Thomas algorithm to solve the resulting system equations.
%Different boundary conditions are allowed at y=y0+H as well as different
%variants of the SA model and different grid nodes distributions.
%There is no error check on the input arguments
%Input*********************************************************************

%Fluid properties
%mu:  dynamic viscosity
%rho: constant density
%Cp:  constant pressure specific heat
%Pr:  molecular Prandtl number

%Flow parameters
%dpdx: pressure gradient
%qdot: volumetric heat source
%y0:   lower boundary
%H:    domain height

%Boundary conditions
%fbcH:  flow and temperature bc flag at y=y0+H, 0->assigned, 1-> symmetry
%UH,TH: values for fbcH=0
%U0,T0: flow and temperature bc values at y=y0
%sabcH: Spalart-Allmaras     bc flag at y=y0+H, 0->assigned, 1-> symmetry
%nuH:   Spalart-Allmaras bc value for sabcH=0

%Spalart-Allmaras/Turbulence parameters
%nu0:   SA initialization value (otherwise SA sources won't kick in)
%Prt:   Turbulent Prandtl number (for the temperature computation)
%vk:    Von Karman constant
%a0:    SA variant selector, 0-> standard SA, a0>0 -> Musker-like SA
%iwall: 0->wall only for y=y0, 1-> both boundaries are walls

%Grid parameters
%method: uniform, sin, gratio, gaussian, tanh, stretched
%par:    parameter for gratio, gaussian, tanh and stretched
%nn:     number of grid nodes (= number of cells + 1)

%Algorithm parameters:
%nit:   max number of iterations
%toll:  convergence tollerance
%saurf: under-relaxation factor for SA equation

%Output********************************************************************
%yc:   grid cell centers
%yn:   grid nodes
%dist: wall distance
%u:    velocity
%t:    temperature
%nut:  SA variable
%mut:  turbulent viscosity
%res:  residual history
%it:   final number of iterations
%**************************************************************************

nu=mu/rho;

%SA constants and parameters
global cv13 nfv1
if (a0==0)
    %Classical SA
    nfv1=3;
    cv13=7.1^nfv1;
else
    %Musker-like SA
    nfv1 = 2;
    cv13=((vk*a0)^(nfv1+1)-(vk*a0)^nfv1);
end
sigma=2/3;
cb1=0.1355;
cb2=0.622;
cw1=cb1/vk^2+(1+cb2)/sigma;
cw2=0.3;
cw3=2;
cv2=0.7;
cv3=0.9;
rlim=10;

%Grid and grid coefficients
nc=nn-1;
yn=newgrid(y0,nn,H,method,par);
dy=diff(yn);
yc=yn(1:end-1)+dy/2;
hp=zeros(nc,1);
hm=zeros(nc,1);
hm(1)=2/(dy(1)*dy(1));
hm(2:nc)=2./(dy(2:nc).*(dy(2:nc)+dy(1:nc-1)));
hp(1:nc-1)=2./(dy(1:nc-1).*(dy(1:nc-1)+dy(2:nc)));
hp(nc)=2/(dy(nc)*dy(nc));
rp=zeros(nc,1);
rm=zeros(nc,1);
rm(1)=1/(yc(1)-yn(1));
rm(2:nc)=1./(yc(2:nc)-yc(1:nc-1));
rp(1:nc-1)=1./(yc(2:nc)-yc(1:nc-1));
rp(nc)=1/(yn(nn)-yc(nc));
if (iwall==0)
    dist=yc-yn(1);
else
    dist=min(abs(yc-yn(1)),abs(yc-yn(nn)));
end
id2=1./dist.^2;

%Initializations
nut=zeros(nc,1)+nu0;
a=zeros(nc,1);
b=zeros(nc,1);
c=zeros(nc,1);
d=zeros(nc,1);
res=ones(nit,1);
it=0;

%Solve iteratively Momentum + SA segregated
while (it<nit)
    %Update turbulent viscosity
    mut=mu*samutmu(nut/nu);

    %Solve momentum********************************************************
    %Matrix coefficients and RHS
    for i = 1 : nc
        switch i
            case 1
                mutp=(mut(i)+mut(i+1))/2;
                mutm=0;
            case nc
                if (sabcH==0)
                    mutp=mu*samutmu(nuH/nu);
                else
                    mutp=mut(i);
                end
                mutm=(mut(i)+mut(i-1))/2;
            otherwise
                mutp=(mut(i)+mut(i+1))/2;
                mutm=(mut(i)+mut(i-1))/2;
        end
        cp=(mu+mutp)*hp(i);
        cm=(mu+mutm)*hm(i);

        a(i)=cm;
        b(i)=-(cp+cm);
        c(i)=cp;
        d(i)=dpdx;
    end
    %Boundary conditions
    d(1) = d(1) - a(1)*U0;
    if (fbcH==0)
        d(nc) = d(nc) - c(nc)*UH; 
    else
        b(nc) = b(nc) + c(nc);
    end
    %Solution
    u = tdma(a,b,c,d);
    %**********************************************************************

    %Solve Spalart-Allmaras************************************************
    %Matrix coefficients and RHS
    for i = 1 : nc
        switch i
            case 1
                dp=(nut(i)+nut(i+1))/2;
                dm=0;
                dudy = (u(i)-U0)/(yc(i)-yn(1));
            case nc
                if (sabcH==0)
                    dp= nuH;
                else
                    dp=nut(i);
                end
                dm=(nut(i)+nut(i-1))/2;
                if (fbcH==0)
                    dudy = (UH-u(i))/(yn(nn)-yc(i));
                else
                    dudy = 0;
                end
            otherwise
                dp=(nut(i)+nut(i+1))/2;
                dm=(nut(i)+nut(i-1))/2;
                dudy = (u(i+1)-u(i-1))/(yc(i+1)-yc(i-1));
        end
        cp=(1+cb2)*(nu+dp)*hp(i)/sigma-(cb2/sigma)*(nu+nut(i))*rp(i);
        cm=(1+cb2)*(nu+dm)*hm(i)/sigma-(cb2/sigma)*(nu+nut(i))*rm(i);

        %Diffusion coefficients
        a(i)=cm;
        b(i)=-(cp+cm);
        c(i)=cp;

        %Source term coefficients and RHS
        chi=nut(i)/nu;
        fv1=safv1(chi);
        fv2=safv2(chi);
        ss=sqrt(dudy^2);
        ds=(nu/vk^2)*id2(i)*chi*fv2;
        if (ds>=-cv2*ss)
            Stilda=ss+ds;
        else
            Stilda=ss+ss*(cv3*ds+cv2^2*ss)/((cv3-2*cv2)*ss-ds);
        end
        Stilda=max(Stilda,1e-14);
        rr=min(rlim,(nu/vk^2)*id2(i)*chi/Stilda);
        gg=rr+cw2*(rr^6-rr);
        ff=(1+cw3^6)/(gg^6+cw3^6);
        fw=gg*ff^(1/6);
        dsdnu=(id2(i)/vk^2)*(fv2+chi*(nfv1*chi*fv1*(1-fv1)-1)/(1+chi*fv1)^2);
        sou=cw1*fw*nut(i)*id2(i)-cb1*Stilda;
        if (rr<rlim)
            dfdg=(cw3^6/(1+cw3^6))*ff^(7/6);
            dgdr=1+cw2*(6*rr^5-1);
            drdnu=(1-nut(i)*dsdnu/Stilda)*id2(i)/(vk^2*Stilda);
            dfdnu=dfdg*dgdr*drdnu;
        else
            dfdnu=0;
        end
        dsou=cw1*id2(i)*(fw+nut(i)*dfdnu)-cb1*dsdnu;
        bb=max(sou,0)+nut(i)*max(dsou,0);
        d(i) = nut(i)*(sou-bb);
        b(i) = b(i) - bb;
    end
    %Boundary conditions
    if (sabcH==0)
        d(nc) = d(nc) - c(nc)*nuH; 
    else
        b(nc) = b(nc) + c(nc);
    end
    %Solution
    nutold = nut;
    nut = nutold+saurf*(tdma(a,b,c,d)-nutold);
    res(it+1)=norm(nut-nutold);
    %**********************************************************************

    %Next iteration
    it = it + 1;

    %Check convergence
    if (res(it)<toll)
        res=res(1:it);
        break
    end
end

%Solve temperature*********************************************************
%Matrix coefficients and RHS
for i = 1 : nc
    switch i
        case 1
            mutp=(mut(i)+mut(i+1))/2;
            mutm=0;
        case nc
            if (sabcH==0)
                mutp=mu*samutmu(nuH/nu);
            else
                mutp=mut(i);
            end
            mutm=(mut(i)+mut(i-1))/2;
        otherwise
            mutp=(mut(i)+mut(i+1))/2;
            mutm=(mut(i)+mut(i-1))/2;
    end
    cp=Cp*(mu/Pr+mutp/Prt)*hp(i);
    cm=Cp*(mu/Pr+mutm/Prt)*hm(i);

    a(i)=cm;
    b(i)=-(cp+cm);
    c(i)=cp;
    d(i)=-qdot;
end
%Boundary conditions
d(1) = d(1) - a(1)*T0;
if (fbcH==0)
    d(nc) = d(nc) - c(nc)*TH;
else
    b(nc) = b(nc) + c(nc);
end
%Solution
t = tdma(a,b,c,d);
%**************************************************************************
end

%Other functions***********************************************************

function fv1 = safv1(chi)
    global cv13 nfv1
    fv1 = chi.^nfv1./(chi.^nfv1+cv13);
end

function mutmu = samutmu(chi)
    mutmu = chi.*safv1(chi);
end

function fv2 = safv2(chi)
    fv2 = 1-chi./(1+chi.*safv1(chi));
end

function [x] = newgrid(x0,nx,L,method,par)
switch method
    case 'sin'
        x=x0+(L/2)*(1+sin(-pi/2+((1:nx)-1)*pi/(nx-1)));
    case 'gratio'
        L1=L*(1-par)/(1-par^(nx-1));
        dx=L1*par.^(0:nx-2);
        x=[x0 x0+cumsum(dx)];
    case 'gaussian'
        b=50;
        par=min(max(0,par),1);
        n=(nx+1)/2-1/2;
        dx=exp(-(((1:nx-1)-n).^2)*b*(par-0.5)/(nx-1)^2);
        dx=L*dx/(sum(dx));
        x=[x0 x0+cumsum(dx)];
    case 'tanh'
        gamma=par;
        x=x0+(L/2)+(L/(2*tanh(gamma)))*tanh(gamma*(2*(0:nx-1)/(nx-1)-1));
    case 'stretched'
        b=max(par,1+eps);
        a=(b+1)/(b-1);
        n=nx-1;
        x=x0+L*0.5*(b-1)*(a.^(2*(0:n)/n)-1)./(a.^(2*(0:n)/n-1)+1);
    otherwise %uniform
        dx=L/(nx-1);
        x=x0:dx:x0+L;
end
end

function [x] = tdma(a,b,c,d)
n = numel(d);
x = zeros(n,1);
for i = 1:n
    if i == 1
        c(i) = c(i)/b(i);
        d(i) = d(i)/b(i);  
    else
        den = b(i) - a(i)*c(i-1);
        c(i) = c(i)/den;
        d(i) = (d(i) - a(i)*d(i-1))/den;
    end
end
for i = n:-1:1
    if i == n
        x(i) = d(i);
    else
        x(i) = d(i) - c(i)*x(i+1); 
    end
end
end