%Script to visualize the iterative procedure to determine tau_wall for a
%given wall function
clear all
close all
clc

%Choose wall function method
% 0: Standard analytical
% 1: Musker Analytical
% 2: Musker Numerical
% 3: Spalart-Allmaras
% 4: Mixing length   (y^3)
% 5: Mixing length 2 (y^4)
% 6: Reichardt intended turbulent viscosity ratio
iwf=0;
switch iwf
    case 0
        a0   = 10.224;
        vk0  = 0.384;
    case 1
        a0   = 10.306;
        vk0  = 0.384;  
    case 2
        a0    = 10.306;
        vk0   = 0.384;        
        mutmu = @(y,a,vk) (vk*y).^3./((vk*y).^2+(vk*a)^3-(vk*a)^2);
    case 3
        vk0   = 0.41;
        a0    = 4.622838269107443/vk0;
        mutmu = @(y,a,vk) (vk*y).^4./((vk*y).^3+(vk*a)^4-(vk*a)^3+256*eps);
    case 4
        a0    = 15.24;
        vk0   = 0.384;        
        mutmu = @(y,a,vk) (vk*y).*(1-exp(-y/a)).^2;
    case 5
        a0    = 15.24;
        vk0   = 0.384;        
        mutmu = @(y,a,vk) (sqrt(1+4*(vk*y).^2.*(1-exp(-y/a)).^2)-1)/2;
    case 6
        a0    = 11;
        vk0   = 0.384;        
        mutmu = @(y,a,vk) vk*(y-a*atan(y/a));
    otherwise
        error('Wrong wall function model')
end

%Flow parameters
U0   = 27.58;
Uw   = 0;
d    = 1;
rho  = 1;
mu   = 1/8000;
dpdx = 0;
%**************************************************************************

Re = rho*(U0-Uw)*d/mu;
Fu = rho*dpdx*d^3/mu^2;
cypl = abs(Fu)^(2/3);
cut  = (mu*abs(dpdx)/rho^2)^(2/3);

if (iwf==0)
    [ait,fait,nit] = spiter(Re,Fu,cypl,@(x) standardsp(x,a0,vk0,0,1));
elseif (iwf==1)
    [ait,fait,nit] = spiter(Re,Fu,cypl,@(x) muskersp(x,a0,vk0,0,1));
else
    [ait,fait,nit] = spiter(Re,Fu,cypl,@(x) numericalsp(x,@(y) mutmu(y,a0,vk0),0,1));
end

disp(strcat('Re:',num2str(Re)))
disp(strcat('Fu:',num2str(Fu)))
disp(strcat('N. iterations:',int2str(nit)))
disp(strcat('Theta:',num2str(ait(nit))))
disp(strcat('tau_wall:',num2str(ait(nit)*mu^2/(rho*d^2))))
disp(strcat('y+:',num2str(sqrt(abs(ait(nit))))))

n    = 500;
amax = max(abs(ait));
alpha = (-amax:2*amax/(2*n-1):amax);
f = zeros(size(alpha));

for i = 1:length(alpha)
    ypl = sqrt(abs(alpha(i))+cypl);
    if (iwf==0)
        [s,p,q] = standardsp(ypl,a0,vk0,0,1);
    elseif (iwf==1)
        [s,p,q] = muskersp(ypl,a0,vk0,0,1);
    else
        [s,p,q] = numericalsp(ypl,@(y) mutmu(y,a0,vk0),0,1);
    end
    f(i) = (Re-Fu*s(2))/s(1);
end

figure()
plot(alpha,alpha,'k',alpha,f,'r')
hold on
plot(ait(1),ait(1),'ks')
plot(ait(2:nit-1),ait(2:nit-1),'bx')
plot(ait(nit),ait(nit),'ro')
hold off
xlabel('\theta')
legend('\theta','f(\theta)=[Re-Fu*s(0)]/s(-1)]','Initialization','Iterations','Final value','Location','Best')

figure()
plot(alpha,f-alpha,'r')
hold on
plot(ait(1),fait(1)-ait(1),'ks')
plot(ait(2:nit-1),fait(2:nit-1)-ait(2:nit-1),'bx')
plot(ait(nit),fait(nit)-ait(nit),'ro')
hold off
xlabel('\theta')
legend('f(\theta)-\theta','Initialization','Iterations','Final value','Location','Best')

nu = 10000;
u=zeros(nu,1);
y=(0:d/(nu-1):d);

tw = mu^2*ait(nit)/(rho*d^2);
ut = sqrt(abs(tw)/rho+cut);

for i = 1:length(y)
    ypl = rho*y(i)*ut/mu;
    if (iwf==0)
        [s,p,q] = standardsp(ypl,a0,vk0,0,1);
    elseif (iwf==1)
        [s,p,q] = muskersp(ypl,a0,vk0,0,1);
    else
        [s,p,q] = numericalsp(ypl,@(y) mutmu(y,a0,vk0),0,1);
    end
    u(i) = Uw + (tw*s(1) + dpdx*y(i)*s(2))*y(i)/mu;
end

figure()
semilogx(y*ut*rho/mu,u/ut)
xlabel('y^+')
ylabel('U^+')

figure()
plot(y,u)
xlabel('y')
ylabel('U(y)')
